//MyTga.cpp
#include "MyTga.h"

MyTga::MyTga() : m_pBuff(NULL), m_pHeader(NULL), m_pData(NULL)
{
}
//---------------------------------------------------------------------------
MyTga::~MyTga()
{
	Release();
}
//---------------------------------------------------------------------------
void MyTga::Release()
{
	if(m_pBuff)
		delete [] m_pBuff;
	if(m_pHeader)
		delete  m_pHeader;
	if(m_pData)
		delete [] m_pData;

	m_pBuff = NULL;
	m_pHeader = NULL;
	m_pData = NULL;
}
//---------------------------------------------------------------------------
bool MyTga::Load( const char* szFileName)
{
	std::ifstream	file;

	file.open(szFileName, std::ios::in | std::ios::binary);
	if(file.fail())
		return false;
	
	file.seekg(0, std::ios::end);
	long flen = file.tellg();
	file.seekg(0, std::ios::beg);

	char* pBuff = new char[flen + 1];
	file.read(pBuff, flen);
	file.close();

	m_pBuff = pBuff;
	if(LoadHeader(pBuff) == false)
	{
		file.close();
		return false;
	}

	pBuff += sizeof( TGAHEADER );
	LoadData(pBuff);

	if(m_pBuff)
	{
		delete [] m_pBuff;
		m_pBuff = NULL;
	}

	return true;
}
//---------------------------------------------------------------------------
bool MyTga::LoadHeader(const char* pBuff)
{
	TGAHEADER* pTgaHeader;
	pTgaHeader = (TGAHEADER *)pBuff;

	m_pHeader = new TGAHEADER;
	memcpy(m_pHeader, pTgaHeader, sizeof(TGAHEADER));

	if(m_pHeader->pixelDepth == 24  || m_pHeader->pixelDepth == 32)
	{
		return true;
	}
	else
	{
		delete m_pHeader;
		m_pHeader = NULL;
		return false;
	}
}
//---------------------------------------------------------------------------
bool MyTga::LoadData(const char* pBuff)
{
	unsigned char colorSwap;
	int colorMode = m_pHeader->pixelDepth / 8;
	int imageSize = m_pHeader->width * m_pHeader->height * colorMode;

	m_pData = new unsigned char [imageSize];
	memcpy(m_pData, pBuff, imageSize);

	//BGR RGB ü
	if(3 == colorMode || 4 == colorMode)
	{
		for(int i = 0; i < imageSize; i += colorMode)
		{
			colorSwap = m_pData[i];
			m_pData[i] = m_pData[i + 2];
			m_pData[i + 2] = colorSwap; 
		}

		return true;
	}
	else
	{
		return false;
	}
}
//---------------------------------------------------------------------------
bool MyTga::Save(const char* szFileName)
{
	if(m_pHeader == NULL || m_pData == NULL)
		return false;

	if(m_pHeader->pixelDepth == 24 || m_pHeader->pixelDepth == 32)
	{
		std::ofstream	file;
		file.open(szFileName, std::ios::out | std::ios::binary);
		if(file.fail())
		{
			file.close();
			return false;
		}

		if(SaveHeader(file) == false)
		{
			file.close();
			return false;
		}
		SaveData(file);
	}

	return true;
}
//---------------------------------------------------------------------------
bool MyTga::SaveHeader(std::ofstream& file)
{
	file.write((const char*)m_pHeader, sizeof(TGAHEADER));
	return true;
}
//---------------------------------------------------------------------------
bool MyTga::SaveData(std::ofstream& file)
{
	int colorMode = m_pHeader->pixelDepth / 8;
	int imageSize = m_pHeader->width * m_pHeader->height * colorMode;

	unsigned char* pData = new unsigned char [imageSize];

	if(3 == colorMode)
	{
		for( int i = 0; i < imageSize; i += colorMode)
		{
			pData[i] = m_pData[i + 2];
			pData[i + 1] = m_pData[i + 1];
			pData[i + 2] = m_pData[i]; 
		}
	}
	else if(4 == colorMode)
	{
		for( int i = 0; i < imageSize; i += colorMode)
		{
			pData[i] = m_pData[i + 2];
			pData[i + 1] = m_pData[i + 1];
			pData[i + 2] = m_pData[i]; 
			pData[i + 3] = m_pData[i + 3];
		}
	}

	file.write((const char*)pData, imageSize);
	return true;
}
//---------------------------------------------------------------------------
const TGAHEADER* MyTga::GetHeader()
{
	return m_pHeader;
}
//---------------------------------------------------------------------------
void MyTga::SetHeader(int bitDepth, int iWidth, int iHeight)
{
	if(m_pHeader == NULL)
	{
		m_pHeader = new TGAHEADER;
		memset(m_pHeader, 0, sizeof(TGAHEADER));
		m_pHeader->imageType = 2;  //  RGB ̹
	}
	m_pHeader->pixelDepth = bitDepth;
	m_pHeader->width = iWidth;
	m_pHeader->height = iHeight;

	if(m_pData)
	{
		delete [] m_pData;
	}

	int colorMode = m_pHeader->pixelDepth / 8;
	int imageSize = m_pHeader->width * m_pHeader->height * colorMode;
	m_pData = new unsigned char [imageSize];
	memset(m_pData, 0, imageSize);
}
//---------------------------------------------------------------------------
unsigned char* MyTga::GetImageData()
{
	return m_pData;
}
//---------------------------------------------------------------------------
